#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "a_station.h"


struct station* create_station() {
    struct station* new_station = malloc(sizeof(struct station));
    if (new_station == NULL) return NULL;

    new_station->tracks = calloc(STATION_SIZE, sizeof(struct car*));
    if (new_station->tracks == NULL) {
        free(new_station);
        return NULL;
    }

    new_station->track_count = STATION_SIZE;
    return new_station;
}

void destroy_station(struct station* station) {
    for (int i = 0; i < station->track_count; i++) {
        struct car* current = station->tracks[i];
        while (current != NULL) {
            struct car* temp = current;
            current = current->next;
            free(temp);
        }
    }
    free(station->tracks);
    free(station);
}


int select_track(struct station* station, const char* target) {
    unsigned long hash = 5381;
    while (*target)
        hash = ((hash << 5) + hash) + *target++;
    return hash % station->track_count;
}


void add_target_capacity(struct station* station, const char* target, int capacity) {
    int index = select_track(station, target);
    struct car* current = station->tracks[index];


    while (current != NULL) {
        if (strcmp(current->value, target) == 0) {
            current->capacity += capacity;
            return;
        }
        current = current->next;
    }

    struct car* new_car = malloc(sizeof(struct car));
    if (new_car == NULL) return;
    strcpy(new_car->value, target);
    new_car->capacity = capacity;
    new_car->next = station->tracks[index];
    station->tracks[index] = new_car;
}

int get_target_capacity(struct station* station, const char* target) {
    int index = select_track(station, target);
    struct car* current = station->tracks[index];

    while (current != NULL) {
        if (strcmp(current->value, target) == 0) {
            return current->capacity;
        }
        current = current->next;
    }
    return 0;
}

int count_targets(struct station* station) {
    int count = 0;
    for (int i = 0; i < station->track_count; i++) {
        struct car* current = station->tracks[i];
        while (current != NULL) {
            count++;
            current = current->next;
        }
    }
    return count;
}

int count_capacity(struct station* station) {
    int total_capacity = 0;
    for (int i = 0; i < station->track_count; i++) {
        struct car* current = station->tracks[i];
        while (current != NULL) {
            total_capacity += current->capacity;
            current = current->next;
        }
    }
    return total_capacity;
}
